"""Declares :class:`AortaEventarcReceiver`."""
import aorta

from cbra.exceptions import Unserializable
from .eventarcreceiver import EventarcReceiver
from .models import MessagePublished


class AortaEventarcReceiver(EventarcReceiver):
    """A :class:`EventarcReceiver` implementation that uses the :mod:`aorta`
    framework.
    """
    __module__: str = 'cbra.ext.google'
    receiver_name: str = "eventarc.googleapis.com"

    def get_correlation_id(self, message) -> str:
        """Return the correlation id for the message, or ``None``."""
        return message.metadata.correlation_id

    def get_message(self, message: MessagePublished):
        try:
            dto = aorta.parse(message.message.get_data())
        except aorta.UnknownMessageType:
            raise Unserializable
        self.logger.info(
            "Received %s/%s (id: %s, correlationId: %s)",
            dto.api_version,
            dto.kind,
            dto.metadata.id,
            dto.metadata.correlation_id
        )
        return dto

    def get_message_handlers(self, message) -> list:
        return aorta.match(message)
