"""Declares :class:`PubsubMessage`."""
import datetime
import binascii
import base64
import json
import typing

import pydantic

from cbra.exceptions import Unserializable


class PubsubMessage(pydantic.BaseModel):
    message_id: str = pydantic.Field(..., alias='messageId')
    publish_time: datetime.datetime = pydantic.Field(..., alias='publishTime')
    attributes: dict = None
    data: str = None

    def get_data(self) -> typing.Union[dict, list]:
        """Return a dictionary or a list containing the message
        data as specified by the ``.data`` attribute. The encoding
        is assumed JSON/UTF-8.
        """
        if self.data is None: # pragma: no cover
            return None
        try:
            return json.loads(base64.b64decode(self.data))
        except (binascii.Error, json.decoder.JSONDecodeError):
            raise Unserializable
