# pylint: skip-file
import base64
import json

import aorta
import pytest

from cbra.exceptions import Unserializable
from cbra.ext.google import AortaEventarcReceiver
from cbra.ext.google.models import MessagePublished


class TestMalformedMessage:

    @pytest.fixture
    def command_b64(self):
        return '['


    def test_malformed_raises_unserializable(self, command_message):
        receiver = AortaEventarcReceiver()
        message = MessagePublished(**json.loads(command_message))
        with pytest.raises(Unserializable):
            receiver.get_message(message)


class TestUnknownMessage:

    @pytest.fixture
    def command_b64(self, command):
        message = command.as_envelope()
        message.kind = "Foo"
        return bytes.decode(base64.b64encode(bytes(message)), 'ascii')

    @pytest.fixture
    def message(self, command_message):
        return command_message

    @pytest.fixture
    def envelope(self, message):
        return MessagePublished(**json.loads(message))

    def test_unknown_raises_unserializable(self, envelope):
        receiver = AortaEventarcReceiver()
        with pytest.raises(Unserializable):
            receiver.get_message(envelope)


class TestInvalidEnvelope:

    @pytest.fixture
    def command_b64(self):
        return bytes.decode(base64.b64encode(b'{}'), 'ascii')

    @pytest.fixture
    def message(self, command_message):
        return command_message

    @pytest.fixture
    def envelope(self, message):
        return MessagePublished(**json.loads(message))

    def test_unknown_raises_unserializable(self, envelope):
        receiver = AortaEventarcReceiver()
        with pytest.raises(Unserializable):
            receiver.get_message(envelope)


