# pylint: skip-file
import pytest


class TestEventArcReceiver:

    @pytest.mark.asyncio
    async def test_missing_principal_is_rejected(self, client):
        response = await client.post('/eventarc/forbidden', json={'foo': 'bar'})
        assert response.status_code == 401, response.text

    @pytest.mark.asyncio
    async def test_invalid_message_is_rejected(self, client):
        response = await client.post('/eventarc', json={'foo': 'bar'})
        assert response.status_code == 422, response.text

    @pytest.mark.asyncio
    async def test_pubsub_message_is_accepted(self, client):
        response = await self.valid_request(client, '/eventarc')
        assert response.status_code == 202, response.text

    @pytest.mark.asyncio
    async def test_missing_principal_is_rejected(self, client):
        response = await self.valid_request(client, '/eventarc/forbidden')
        assert response.status_code == 401, response.text

    @pytest.mark.asyncio
    async def test_invalid_email_is_rejected(self, client):
        response = await self.valid_request(client, '/eventarc/wrong-email')
        assert response.status_code == 403, response.text

    @pytest.mark.asyncio
    async def test_no_members_defined_is_rejected(self, client):
        response = await self.valid_request(client, '/eventarc/no-members')
        assert response.status_code == 403, response.text

    async def valid_request(self, client, url):
        return await client.post(url, json={
			"subscription": "projects/myproject/subscriptions/mysub",
			"message": {
				"messageId": "string",
				"publishTime": "2022-03-12T16:48:01.313Z",
				"attributes": {},
				"data": "string"
			}
        })
