# pylint: skip-file
import jwt
import pytest


@pytest.skip(allow_module_level=True)
class TestGoogleIAM:

    @pytest.mark.asyncio
    async def test_endpoint_accepts_valid_google_token_get(self, client):
        response = await client.get('/protected')
        assert response.status_code == 200, response.json()

    @pytest.mark.asyncio
    async def test_endpoint_rejects_invalid_audience(self, client):
        response = await client.get('/invalid-audience')
        assert response.status_code == 403, response.json()
        error = response.json()
        assert 'spec' in error
        assert 'code' in error['spec']
        assert error['spec']['code'] == 'INVALID_AUDIENCE'

    @pytest.mark.asyncio
    async def test_endpoint_rejects_invalid_email(self, client):
        response = await client.get('/invalid-service-account')
        assert response.status_code == 403, response.json()
        error = response.json()
        assert 'spec' in error
        assert 'code' in error['spec']
        assert error['spec']['code'] == 'FORBIDDEN'

    @pytest.mark.asyncio
    async def test_endpoint_rejects_unknown_signer(self, client, keychain):
        key = keychain.get('rsa')
        token = jwt.encode({}, await key.get_secret(), algorithm='RS256')
        headers = {'Authorization': f"Bearer {token}"}
        response = await client.get('/protected', headers=headers)
        assert response.status_code == 403, response.json()
        error = response.json()
        assert 'spec' in error
        assert 'code' in error['spec']
        assert error['spec']['code'] == 'INVALID_SIGNATURE'
