# pylint: skip-file
import pytest
from fastapi import Request

from ..googlejwtprincipal import GoogleJWTPrincipal


class TestGoogleJWTPrincipal:

    def request_factory(self, host, path):
        return Request(
            {
                'type': 'http',
                'path': path,
                'scheme': 'http',
                'headers': [
                    (b'host', str.encode(host))
                ]
            }
        )

    @pytest.mark.asyncio
    async def test_no_audience_specified(self):
        request = self.request_factory('foo.com', '/foo')
        principal = GoogleJWTPrincipal()
        assert principal.get_allowed_audiences(request) == {"http://foo.com"}

    @pytest.mark.asyncio
    async def test_string_audience_specified(self):
        request = self.request_factory('foo.com', '/foo')
        principal = GoogleJWTPrincipal(
            audience="https://bar.com"
        )
        assert principal.get_allowed_audiences(request) == {"https://bar.com"}

    @pytest.mark.asyncio
    async def test_set_audience_specified(self):
        request = self.request_factory('foo.com', '/foo')
        principal = GoogleJWTPrincipal(
            audience={"https://bar.com", "https://baz.com"}
        )
        assert principal.get_allowed_audiences(request) == {"https://bar.com", "https://baz.com"}

    @pytest.mark.asyncio
    async def test_callable_str_audience_specified(self):
        request = self.request_factory('foo.com', '/foo')
        principal = GoogleJWTPrincipal(
            audience=lambda x: "https://bar.com"
        )
        assert principal.get_allowed_audiences(request) == {"https://bar.com"}

    @pytest.mark.asyncio
    async def test_callable_set_audience_specified(self):
        request = self.request_factory('foo.com', '/foo')
        principal = GoogleJWTPrincipal(
            audience=lambda x: {"https://bar.com"}
        )
        assert principal.get_allowed_audiences(request) == {"https://bar.com"}
