"""Declares :class:`AuthorizationCode`."""
import secrets
import typing

import pydantic

from .authorizationrequest import AuthorizationRequest


class AuthorizationCode(pydantic.BaseModel):
    __module__: str = 'cbra.ext.oauth2'
    code: str
    request: AuthorizationRequest
    consumed: bool = False

    @classmethod
    def new(
        cls,
        request: typing.Union[AuthorizationRequest, typing.Dict[str, typing.Any]],
        length: int = 24
    ) -> 'AuthorizationCode':
        """Generate a new authorization code and return
        a :class:`AuthorizationCode` instance.
        """
        return cls(
            code=secrets.token_urlsafe(length),
            request=request
        )