"""Declares :class:`IClientRepository`."""
import abc
import typing

from .exceptions import ClientDoesNotExist
from .iclient import IClient


class IClientRepository(metaclass=abc.ABCMeta):
    """Specifies the interface for :term:`IClient` repository
    implementations.
    """
    __module__: str = 'cbra.ext.oauth2'
    DoesNotExist: typing.Type[BaseException] = ClientDoesNotExist
    ClientDoesNotExist: typing.Type[BaseException] = DoesNotExist

    @abc.abstractmethod
    async def exists(self, client_id: str) -> bool:
        """Retur a boolean indicating if the client exists."""
        raise NotImplementedError

    @abc.abstractmethod
    async def get(self, client_id: str) -> IClient:
        """Lookup an OAuth 2.0 client using its client identifier."""
        raise NotImplementedError