"""Declares :class:`MemoryStorage`."""
import typing

from ckms.jose import ClaimSet

from .istorage import IStorage


class MemoryStorage(IStorage):
    """A :class:`~cbra.ext.oauth2.IStorage` implementation
    that used the local memory. For testing purposes only.
    """
    __module__: str = 'cbra.ext.oauth2'
    objects: typing.Dict[str, typing.Union[bytes, str]] = {}

    def __init__(self):
        self.objects = MemoryStorage.objects

    @classmethod
    def clear(self) -> None:
        MemoryStorage.objects = {}

    async def consume(self, claims: ClaimSet) -> None:
        """Consume the claims set, ensuring that its token can only be
        used once. Return a boolean indicating if the token was already
        consumed.
        """
        k = f'consumed.{claims.jti}'
        is_consumed = True
        if k not in self.objects:
            self.objects[k] = ...
            is_consumed = False
        return is_consumed