# pylint: skip-file
import typing

import fastapi
import pydantic

from cbra.ext import ioc
from .iclientrepository import IClientRepository
from .isubjectrepository import ISubjectRepository
from .iopenauthorizationserver import IOpenAuthorizationServer
from .istorage import IStorage


__all__ = [
    'ClientRepository',
    'CurrentServerMetadata',
    'LocalIssuer',
    'TransientStorage',
    'Server',
    'ServerCodec',
    'ServerJWKS',
    'SubjectRepository',
]

def _get_server(request: fastapi.Request) -> IOpenAuthorizationServer:
    return request.scope['oauth2'] # type: ignore


def get_local_issuer(request: fastapi.Request) -> str:
    return f'{request.url.scheme}://{request.url.netloc}'


ClientRepository: IClientRepository = ioc.instance(
    'cbra.ext.oauth2.ClientRepository'
)

Server = fastapi.Depends(_get_server)


async def get_server_metadata(
    request: fastapi.Request,
    server: IOpenAuthorizationServer = Server
) -> typing.Any:
    return await server.metadata_endpoint(request)

CurrentServerMetadata = fastapi.Depends(get_server_metadata)

LocalIssuer = fastapi.Depends(get_local_issuer)

ServerCodec = ioc.inject(
    'cbra.ext.oauth2.ServerCodec'
)

ServerJWKS = ioc.inject(
    'cbra.ext.oauth2.ServerJWKS'
)

SubjectRepository: ISubjectRepository = ioc.instance(
    'cbra.ext.oauth2.SubjectRepository'
)

TransientStorage: IStorage = ioc.instance(
    'cbra.ext.oauth2.TransientStorage'
)