# pylint: skip-file
"""
.. _ref-oauth2-reference-server

==================================
OAuth 2.0 reference implementation
==================================
"""
import collections

import ckms

from . import clients
from . import keys
from . import subject


__all__ = [
    'DEFAULT_ENCRYPTION_KEY',
    'DEFAULT_SIGNING_KEY',
    'alice',
    'bob',
    'clients',
    'keys',
    'keychain',
    'subject'
]

DEFAULT_ENCRYPTION_KEY = 'enc'

DEFAULT_SIGNING_KEY = 'sig'

alice = subject.Subject(
    sub="alice@example.unimatrixone.io",
    client_scope=collections.defaultdict(set, {
        'jwt': {"read", "write"}
    }),
    keys=[keys.alice]
)

bob = subject.Subject(
    sub="bob@example.unimatrixone.io",
    client_scope=collections.defaultdict(set, {
        'jwt': {"read", "write"},
        'confidential': {"read"}
    }),
    keys=[keys.bob]
)

keychain: ckms.Keychain = ckms.Keychain()
keychain.register({
    DEFAULT_ENCRYPTION_KEY: {
        'provider': 'local',
        'kty': "OKP",
        'crv': 'Ed448',
        'tags': ['oauth2'],
        'use': 'enc',
    },
    DEFAULT_SIGNING_KEY: {
        'provider': 'local',
        'kty': "OKP",
        'crv': 'Ed448',
        'tags': ['oauth2'],
        'use': 'sig',
    },
})