# pylint: skip-file
import collections
import dataclasses
import typing

from ckms.jose import JSONWebSignature
from ckms.jose.models import JSONWebKeySet
from ckms.jose.models import JSONWebKey

from cbra.ext.oauth2 import ISubject


@dataclasses.dataclass
class Subject(ISubject):
    sub: str
    client_id: typing.Optional[str] = None
    client_scope: typing.Dict[str, typing.Set[str]] = dataclasses.field(
        default_factory=lambda: collections.defaultdict(set)
    )
    keys: typing.List[JSONWebKey] = dataclasses.field(
        default_factory=list
    )

    @property
    def jwks(self) -> JSONWebKeySet:
        return JSONWebKeySet(keys=self.keys)

    def allows_scope(self, scope: typing.Set[str]) -> bool:
        assert isinstance(scope, set)
        if self.client_id is None:
            return False
        return scope <= self.client_scope[self.client_id]

    def register_public_key(self, key: JSONWebKey) -> None:
        self.keys.append(key)

    def verify(
        self,
        jws: JSONWebSignature
    ) -> typing.Union[bool, typing.Coroutine[typing.Any, typing.Any, bool]]:
        return self.jwks.verify(jws)