"""Declares :class:`SpaceSeparatedList`."""
import re
import typing


class SpaceSeparatedList(set):
    __module__: str = 'cbra.ext.oauth2'
    sep: re.Pattern = re.compile('\s+')

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(
        cls,
        data: typing.Union[str, typing.List[str]]
    ):
        if isinstance(data, (list, set)):
            return set(data)
        return cls([x for x in cls.sep.split(data)]) if data else []

    @classmethod
    def __modify_schema__(
        cls,
        field_schema: typing.Dict[str, typing.Any]
    ):
        field_schema.update(
            type="string",
            example="value1 value2 value3"
        )

    def __str__(self) -> str:
        return str.join(' ', sorted(self))
