"""Declares :class:`TokenRequestHandler`."""
import typing
from cbra.ext.oauth2.tokenresponse import TokenResponse

import fastapi

from cbra.ext import ioc
from .iopenauthorizationserver import IOpenAuthorizationServer
from .requestcontext import RequestContext
from .requesthandler import RequestHandler
from .servermetadata import ServerMetadata
from .tokenissuer import TokenIssuer
from .tokenrequest import TokenRequest
from .tokenrequestclient import TokenRequestClient


class TokenRequestHandler(RequestHandler):
    __module__: str = 'cbra.ext.oauth2'
    response_model: typing.Type[TokenResponse] = TokenResponse

    #: A :class:`~cbra.ext.oauth2.TokenIssuer` instance that is used
    #: to issue tokens in response to requests to the **Token Endpoint**.
    issuer: TokenIssuer

    @classmethod
    def as_handler(
        cls,
        server: IOpenAuthorizationServer,
        metadata: ServerMetadata,
        **kwargs: typing.Dict[str, typing.Any]
    ):
        """Return a request handler that invokes the handler."""

        async def f(
            dto: TokenRequest = TokenRequest.as_dependant(
                client=TokenRequestClient
            ),
            ctx: RequestContext = ioc.instance(RequestContext),
            handler: TokenRequestHandler = ioc.instance(cls),
            issuer: TokenIssuer = ioc.instance(TokenIssuer)
        ) -> TokenResponse:
            kwargs.setdefault('issuer', issuer)
            handler._setup(**kwargs)
            return await handler.handle(ctx, dto)
        f.__doc__ = cls.__doc__
        return f

    def setup(self, issuer: TokenIssuer):
        self.issuer = issuer

    async def handle(
        self,
        ctx: RequestContext,
        dto: TokenRequest
    ):
        return await self.issuer.grant(dto)