"""Declares :class:`InjectedMessageHandler`."""
import inspect

import aorta
import fastapi.params

from .dependencyreceiver import DependencyReceiver


def update_signature(cls, self) -> inspect.Signature:
    signature = cls.__signature__.fget(self)
    params = list(signature.parameters.values())
    params[0] = inspect.Parameter(
        name=params[0].name,
        kind=params[0].kind,
        default=fastapi.params.Body(...),
        annotation=params[0].annotation
    )
    return inspect.Signature(params)


class CommandHandler(aorta.CommandHandler, DependencyReceiver):
    __abstract__: bool = True
    __module__: str = 'cbra'

    @property
    def __signature__(self) -> inspect.Signature:
        return update_signature(aorta.CommandHandler, self)


class EventListener(aorta.EventListener, DependencyReceiver):
    __abstract__: bool = True
    __module__: str = 'cbra'

    @property
    def __signature__(self) -> inspect.Signature:
        return update_signature(aorta.EventListener, self)
