"""Declares :class:`Options`."""
import typing

import fastapi

from cbra.headers import ACCESS_CONTROL_ALLOW_CREDENTIALS_SCHEMA
from cbra.headers import ACCESS_CONTROL_ALLOW_HEADERS_SCHEMA
from cbra.headers import ACCESS_CONTROL_ALLOW_METHODS_SCHEMA
from cbra.headers import ACCESS_CONTROL_ALLOW_ORIGIN_SCHEMA
from cbra.headers import ACCESS_CONTROL_MAX_AGE_SCHEMA
from cbra.utils import classproperty
from .endpoint import Endpoint


class Options(Endpoint):
    __module__: str = 'cbra'
    allowed_methods: typing.Set[str] = set()

    @classproperty
    def default_response(cls) -> typing.Dict[str, typing.Any]:
        return {
            'description': "Endpoint allowed methods and CORS privileges",
            'headers': {
                'Access-Control-Allow-Credentials': ACCESS_CONTROL_ALLOW_CREDENTIALS_SCHEMA,
                'Access-Control-Allow-Headers': ACCESS_CONTROL_ALLOW_HEADERS_SCHEMA,
                'Access-Control-Allow-Methods': ACCESS_CONTROL_ALLOW_METHODS_SCHEMA,
                'Access-Control-Allow-Origin': ACCESS_CONTROL_ALLOW_ORIGIN_SCHEMA,
                'Access-Control-Max-Age': ACCESS_CONTROL_MAX_AGE_SCHEMA
            }
        }
    
    @classmethod
    def add_to_router(
        cls,
        *,
        app: Endpoint.RouterType,
        base_path: str,
        method: str = 'OPTIONS',
        **kwargs: typing.Any
    ) -> None:
        kwargs['response_class'] = fastapi.Response
        return super().add_to_router(
            app=app,
            base_path=base_path,
            method=method,
            **kwargs
        )

    async def handle(self) -> fastapi.Response:
        response = fastapi.Response()
        response.headers['Allow'] = str.join(', ', sorted(self.allowed_methods|{"OPTIONS"}))\
            or "OPTIONS"
        return response