# pylint: skip-file
import pytest
import pytest_asyncio
from httpx import AsyncClient

import cbra


class TestRunner(cbra.query.QueryRunner):

    async def run_test_query(self, query):
        return query.foo

    async def run_test_request_query(self, query):
        return query


class TestQuery(cbra.query.NamedQuery):
    foo: int

    def build(self, foo: int):
        return self.model(foo=foo)


class RequestQuery(cbra.query.RequestQuery):
    foo: int
    bar: int = 2


@pytest.fixture
def query():
    return TestQuery.model(foo=1)


@pytest.fixture
def app():
    app = cbra.Application()
    app.inject('cbra.runner.default', TestRunner)
    query = RequestQuery()

    @app.get('/', dependencies=query.get_request_dependencies())
    async def run_query(query=RequestQuery(run=False)):
        return query.dict()

    return app


@pytest_asyncio.fixture
async def client(app):
    async with AsyncClient(app=app, base_url="http://cbra.internal") as client:
        yield client


@pytest.fixture
def runner(app):
    yield app.container.resolve('cbra.runner.default')
