# pylint: skip-file
import unittest

from fastapi.testclient import TestClient
from unimatrix.exceptions import CanonicalException

from ..application import Application


class TestCanonicalException(unittest.TestCase):

    class exception_class(CanonicalException):
        http_status_code = 511
        code = 'TEST_EXCEPTION'

    def setUp(self):
        self.app = Application(
            allowed_hosts=['*'],
            log_exceptions=True
        )
        self.client = TestClient(self.app)
        self.app.add_api_route(
            '/exception',
            self.raise_canonical
        )

    async def raise_canonical(self):
        raise self.exception_class

    def test_canonical_exception_returns_status_code(self):
        response = self.client.get('/exception')
        self.assertEqual(response.status_code, self.exception_class.http_status_code)

    def test_canonical_exception_returns_header(self):
        response = self.client.get('/exception')
        self.assertIn('X-Canonical-Exception', response.headers)

    def test_canonical_exception_returns_code(self):
        response = self.client.get('/exception')
        dto = response.json()
        self.assertEqual(dto['spec']['code'], self.exception_class.code)
