# pylint: skip-file
import types

import pytest

import cbra


@pytest.fixture
def settings():
    obj = types.ModuleType('settings')
    obj.DEPENDENCIES = [
        {
            'type': "symbol",
            'name': "TestDependency",
            'value': 'cbra.conftest.TEST_DEPENDENCY'
        }
    ]
    return obj


async def test_dependency_is_resolvable(settings):
    app = cbra.Application(settings=settings)
    await app.boot()
    assert app.container.is_satisfied('TestDependency')


async def test_dependency_container_ignores_settings(container, settings):
    assert container is not None
    assert not container.is_satisfied('TestDependency')
    app = cbra.Application(provider=container)
    await app.boot()
    assert app.container == container
    assert not app.container.is_satisfied('TestDependency')


async def test_dependency_explicit_inject_always_overrides(settings, test_dependency):
    app = cbra.Application(settings=settings)
    await app.boot()
    assert app.container.is_satisfied('TestDependency')
    assert app.container.resolve('TestDependency') == test_dependency
    app.inject('TestDependency', "123")
    assert app.container.resolve('TestDependency') == "123"
