# pylint: skip-file
import pytest


class TestContentNegotiation:

    @pytest.mark.asyncio
    async def test_unspecified_content_type_returns_415(self, client):
        response = await client.post(
            '/examples',
            content='{}'
        )
        assert response.status_code == 415
        assert 'Accept' in response.headers, response.headers

    @pytest.mark.asyncio
    async def test_invalid_json_returns_400(self, client):
        response = await client.post(
            '/examples',
            content='{',
            headers={'Content-Type': "application/json"}
        )
        assert response.status_code == 400, response.status_code
        dto = response.json()
        assert 'spec' in dto, dto
        assert dto['spec'].get('code') == 'MALFORMED_BODY'

    @pytest.mark.asyncio
    async def test_valid_json(self, client):
        response = await client.post('/examples', json={'foo': 1, 'bar': 2})
        assert response.status_code == 200

    @pytest.mark.asyncio
    async def test_valid_form(self, client):
        response = await client.post('/examples', data={'foo': '1', 'bar': '2'})
        assert response.status_code == 200
        assert response.json() == {'dto': {'foo': '1', 'bar': '2'}}
