# pylint: skip-file
import fastapi
import pytest

import cbra
from cbra.ext import ioc


@pytest.mark.asyncio
async def test_run_simple(run_dependant):

    async def f(instance=ioc.Instance(InstanceDependency)):
        assert isinstance(instance, InstanceDependency)

    await run_dependant(f)


@pytest.mark.asyncio
async def test_run_ioc(run_dependant):
    ioc.provide('InstanceDependency', InstanceDependency, force=True)

    async def f(instance=ioc.Instance('InstanceDependency')):
        assert isinstance(instance, InstanceDependency)

    await run_dependant(f)


@pytest.mark.asyncio
async def test_run_with_injected_constructor(run_dependant):
    instance = await run_dependant(ioc.Instance(InjectingInstanceDependency))
    assert isinstance(instance, InjectingInstanceDependency)
    assert instance.foo == 1
    assert instance.bar == 2
    assert instance.baz == 3


class InstanceDependency:
    pass


class InjectingInstanceDependency:

    def __init__(self,
        foo = fastapi.Depends(lambda: 1),
        bar = fastapi.Depends(lambda: 2),
        baz = fastapi.Depends(lambda: 3),
    ):
        self.foo = foo
        self.bar = bar
        self.baz = baz
