# pylint: skip-file
import uuid

import pytest

from cbra.ext import ioc
from ..transaction import Transaction


class TestTransactionBehavior:

    @pytest.mark.asyncio
    async def test_publisher_is_invoked_after_transaction(self, transport, command, event):
        async with Transaction() as tx:
            tx.issue(command)
            tx.publish(event)
            assert not transport.send.called
        assert transport.send.called

    @pytest.mark.asyncio
    async def test_publisher_is_not_invoked_after_exception(self, transport, command, event):
        try:
            async with Transaction() as tx:
                tx.issue(command)
                tx.publish(event)
                assert not transport.send.called
                raise Exception
        except Exception:
            pass
        assert not transport.send.called

    @pytest.mark.asyncio
    async def test_transaction_generates_correlation_id(self, transport, command, event):
        async with Transaction() as tx:
            correlation_id = tx.correlation_id
            tx.issue(command)
            tx.publish(event)
        assert len(transport.send.call_args.args) == 1
        for obj in transport.send.call_args.args[0]:
            assert obj.metadata.correlation_id == correlation_id

    @pytest.mark.asyncio
    async def test_transaction_sets_correlation_id(self, transport, command, event):
        correlation_id = uuid.uuid4()
        async with Transaction(correlation_id=correlation_id) as tx:
            assert tx.correlation_id == correlation_id
            tx.issue(command)
            tx.publish(event)
        assert len(transport.send.call_args.args) == 1
        for obj in transport.send.call_args.args[0]:
            assert obj.metadata.correlation_id == correlation_id
