"""Declares :class:`ICorsPolicy`."""
import typing

import fastapi

from cbra.headers import ACCESS_CONTROL_REQUEST_HEADERS
from cbra.headers import ACCESS_CONTROL_REQUEST_METHOD
from cbra.headers import ORIGIN
from cbra.params import CurrentServer


class ICorsPolicy:
    __module__: str = 'cbra.types'
    allow_credentials: bool = False
    allowed_methods: typing.Set[str] = set()
    allowed_headers: typing.Set[str] = set()
    allowed_origins: typing.Set[str] = set()
    max_age: int = 86400
    origin: typing.Optional[str]
    request_headers: typing.Set[str]
    request_methods: typing.Set[str]
    server: str

    @classmethod
    def as_options(
        cls,
        server: str = CurrentServer,
        origin: typing.Optional[str] = ORIGIN,
        request_headers: typing.Optional[str] = ACCESS_CONTROL_REQUEST_HEADERS,
        request_methods: typing.Optional[str] = ACCESS_CONTROL_REQUEST_METHOD
    ) -> 'ICorsPolicy':
        instance = cls(
            server=server,
            origin=origin,
        )
        instance.request_headers = set(
            filter(bool, map(str.strip, str.split(request_headers or '', ',')))
        )
        instance.request_methods = set(
            filter(bool, map(str.strip, str.split(request_methods or '', ',')))
        )
        return instance

    def __init__(
        self,
        server: str = CurrentServer,
        origin: typing.Optional[str] = ORIGIN
    ):
        self.origin = origin
        self.server = server

    async def add_response_headers(
        self,
        request: fastapi.Request,
        response: fastapi.Response,
        handler: typing.Any,
    ) -> None:
        raise NotImplementedError

    async def check_cors_allowed(
        self,
        handler: typing.Any,
        origin: typing.Optional[str]
    ) -> typing.Optional[typing.NoReturn]:
        raise NotImplementedError

    async def process_request(
        self,
        request: fastapi.Request,
        handler: typing.Any,
        origin: typing.Optional[str] = None
    ) -> typing.Union[bool, typing.NoReturn]:
        raise NotImplementedError

    async def process_response(
        self,
        request: fastapi.Request,
        response: fastapi.Response,
        handler: typing.Any,
        origin: typing.Optional[str] = None
    ) -> typing.Optional[typing.NoReturn]:
        raise NotImplementedError