"""Declares :class:`IPrincipal`."""
import typing

from ckms.jose import ClaimSet


class IPrincipal:
    """Specifies the interface for all principal implementations."""
    __module__: str = 'cbra.lib.auth'

    sub: typing.Union[int, str]

    @classmethod
    def fromclaimset(
        cls,
        claims: ClaimSet
    ) -> 'IPrincipal':
        """Instantiate a new :class:`IPrincipal` using the claims provided with a
        JSON Web Token (JWT).
        """
        raise NotImplementedError

    def has_scope(
        self,
        scope: typing.Union[str, typing.Set[str]]
    ) -> bool:
        """Return a boolean indicating if the :class:`IPrincipal` is
        authorized to use the given `scope`.
        """
        raise NotImplementedError